package no.usit.fridaws;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import javax.naming.InitialContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Felles database funksjonalitet for webservicene.
 */
class Database {
    private static final Logger logger = LoggerFactory.getLogger(Database.class);

    /**
     * Returnerer en Connection til databasen webservicene skal bruke.
     * DataSource for basen blir slaatt opp med et hardkodet JNDI navn.
     *
     * @return en database Connection
     */
    public static Connection getConnection() throws java.sql.SQLException {
		DataSource ds;
		try {
		    InitialContext ctxt = new InitialContext();
		    ds = (DataSource) ctxt.lookup("java:/jdbc/FDS");
		    return ds.getConnection();
		} catch (javax.naming.NamingException e) {
		    return null;
		}
    }

    /**
     * Returnerer en persons løpenr, gitt personens brukernavn og eierkode.
     * Hvis personen ikke finnes blir null returnert.
     *
     * @param brukerNavn personens brukernavn hos en 'eier'
     * @param eierKode personens eierkode, f.eks. UIO, UIB, osv.
     * @return personens løpenr
     */
    public static Long getPersonLopeNr(String brukerNavn, String eierKode) {
		Connection conn = null;
		try {
		    eierKode = eierKode.toUpperCase();
	
		    conn = getConnection();
		    PreparedStatement stmt = null;
		    stmt = conn.prepareStatement(
					 "SELECT personlopenr " +
					 "FROM   person p, bruker b " +
					 "WHERE  p.fodselsdato = b.fodselsdato " +
					 "  AND  p.personnr = b.personnr " +
					 "  AND  b.brukernavn = :1 " +
					 "  AND  b.eierkode = :2");
		    stmt.setString(1, brukerNavn);
		    stmt.setString(2, eierKode);

		    ResultSet rs = stmt.executeQuery();
		    if (rs.next())
			return rs.getLong(1);
		} catch (SQLException e) {
		    loggFeil(e);
		} finally {
		    try {
			if (conn != null)
			    conn.close();
		    } catch (SQLException e) {
		    	loggFeil(e);
		    	return null;
		    }
		}
		return null;
    }

    /**
     * Logger en database-feil.
     *
     * @param e SQLException som oppstod
     */
    static void loggFeil(SQLException e) {
    	logger.error("SQL feil: [{}] {}", e.getErrorCode(), e.getMessage());
    }
}
