package no.usit.fridaws;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.sql.DataSource;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Variant;

import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.Date;
import java.util.ArrayList;

import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;

import org.jboss.resteasy.annotations.cache.Cache;

import java.sql.ParameterMetaData;

import java.lang.Integer;

import net.sf.json.xml.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * PresentasjonSted RPC-tjeneste.
 */
@Path("/hentPresentasjonSted")
public class PresentasjonSted {

    private static final Logger logger =
	LoggerFactory.getLogger(Database.class);

    /**
     * Returnerer presentasjonene som er knyttet til et sted.
     *
     * @param instnr stedets institusjonsnr.
     * @param avdnr stedets avdelingsnr.
     * @param undavdnr stedets underavdelingsnr.
     * @param gruppenr stedets gruppenr.
     * @param sortering sorteringsangivelse, se bruker-doc.
     * @param format "xml" eller "json" for å angi output format
     * @param type presentasjonstype
     * @return en liste med presentasjoner
     */
    @GET
    @Cache(maxAge=0)
    public Response hentPresentasjonSted(
		@QueryParam("instnr") Long instnr,
		@QueryParam("avdnr") Long avdnr,
		@QueryParam("undavdnr") Long undavdnr,
		@QueryParam("gruppenr") Long gruppenr,
		@QueryParam("format") String format,
		@QueryParam("type") String type,
		@QueryParam("sortering") String sortering
	) {
		if (format == null) {
		    format = "xml";
		} else {
		    format = format.toLowerCase();
		}
	
		if (type != null) {
		    type = type.toUpperCase();
		}
	
		if ((instnr == null) ||
		    (type == null) ||
		    (!format.equals("xml") && !format.equals("json"))) {
		    return Response.status(400).build();
		}
	
		String output = null;
		Connection conn = null;
		PreparedStatement stmt;
	
		try {
		    conn = Database.getConnection();
		    String sql;
		    ArrayList<Object> params = new ArrayList<Object>();
	
		    sql =
		    	"SELECT x.tekst_xml from " +
				"       presentasjon_xml x, " +
				"       presentasjon_enhet pe, " +
				"       presentasjon p " +
				"WHERE  pe.presentasjonslopenr = p.presentasjonslopenr " +
				"  AND  p.presentasjonslopenr = x.presentasjonslopenr " +
				"  AND  p.presentasjonstypekode = ? " +
				"  AND  pe.institusjonsnr = ? ";
		    params.add(type);
		    params.add(instnr);
	
		    if (avdnr != null) {
				sql += " AND avdnr = ? ";
				params.add(avdnr);
		    }
		    if (undavdnr != null) {
				sql += " AND undavdnr = ? ";
				params.add(undavdnr);
		    }
		    if (gruppenr != null) {
				sql += " AND gruppenr = ? ";
				params.add(gruppenr);
		    }
	
		    if (sortering != null) {
				String sortpred = Filter.lagPresentasjonSortering(sortering);
				if (sortpred == null) {
				    return Response.status(400).build();
				}
				sql += " ORDER BY " + sortpred + " ";
		    }
		    // til slutt, begrens antall: (dette maa gjoeres sist)
		    sql = "select tekst_xml from (" + sql + ") where ROWNUM <= 1001";
	
	
		    logger.debug(sql);
	
		    stmt = conn.prepareStatement(sql,
						 ResultSet.TYPE_SCROLL_INSENSITIVE,
						 ResultSet.CONCUR_READ_ONLY);
		    
		    int paramIndex = 0;
		    for (Object p : params) {
		    	logger.debug("setter inn object #{} med verdi <{}>",
				     paramIndex,
				     p.toString());
		    	stmt.setObject(++paramIndex, p);
		    }
	
		    ResultSet rs = stmt.executeQuery();
	
		    rs.last();
		    logger.debug("rs.getRow() == {}", rs.getRow());
		    if (rs.getRow() > 1000) {
		    	return Response.status(413).build();
		    }
		    rs.first();
	
		    ArrayList<String> parts = new ArrayList<String>();
		    while (rs.next()) {
		    	parts.add(rs.getString(1));
		    }
		    if (parts.size() == 0) {
		    	return Response.status(404).build();
		    }
	
		    logger.debug("antall elementer = {}", parts.size());
	
		    XPathExpression xp = null;
		    try {
				if (type.equals("PROSJEKT")) {
				    xp = Xml.stringToXPath("/frida/prosjekt");
				} else if (type.equals("GRUPPE") ||
						   type.equals("ENHET") ||
						   type.equals("SENTER")) {
					xp = Xml.stringToXPath("/frida/forskningsenhet");
				} else if (type.equals("PERSON")) {
					xp = Xml.stringToXPath("/frida/forskerprofil");
				} else {
					return Response.status(400).build();
				}
	
				output = Xml.join("http://frida.usit.uio.no/"
						  + "xmlSchema/xsd/frida.xsd",
						  xp, parts);
				
		    } catch (Exception e) {
		    	return Response.status(500).build();
		    }
		} catch (SQLException e) {
		    Database.loggFeil(e);
		    return Response.status(500).build();
		} finally {
		    try {
				if (conn != null) {
				    conn.close();
				}
		    } catch (SQLException e) {
				Database.loggFeil(e);
				return Response.status(500).build();
		    }
		}
	
		MediaType outputFormat = MediaType.APPLICATION_XML_TYPE;
	
		if (format.equals("json")) {
		    XMLSerializer xs = new XMLSerializer();
		    output = xs.read(output).toString();
		    outputFormat = MediaType.APPLICATION_JSON_TYPE;
		}
	
		return Response
		    .status(200)
		    .entity(output)
		    .variant(new Variant(outputFormat,
					 new java.util.Locale("no_NO"),
					 null))
		    .build();
    }

}
