package no.usit.fridaws;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.sql.DataSource;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Variant;

import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.Date;
import java.util.ArrayList;

import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;

import org.jboss.resteasy.annotations.cache.Cache;

import java.sql.ParameterMetaData;

import java.lang.Integer;

import net.sf.json.xml.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * VarbeidSted RPC-tjeneste.
 */
@Path("/hentVarbeidSted")
public class VarbeidSted {

    private static final Logger logger =
	LoggerFactory.getLogger(VarbeidSted.class);

    /**
     * Returnerer Varbeidene til et sted.
     *
     * @param instnr stedets institusjonsnr.
     * @param avdnr stedets avdelingsnr.
     * @param undavdnr stedets underavdelingsnr.
     * @param gruppenr stedets gruppenr.
     * @param format "xml" eller "json" for å angi output format
     * @param fra hent kun varbeider utgitt fra og med dette årstallet
     * @param til hent kun varbeider utgitt til og med dette årstallet
     * @param utplukk "alle", "nyeste" eller "viktigste" arbeider
     * @param maksantall begrenser svaret til dette antallet varbeider
     * @param hovedkategori begrens til kun denne hovedkategorien
     * @param underkategori begrens til kun denne underkategorien
     * @param sortering sorteringsangivelse, se bruker-doc.
     * @return en liste med varbeider
     */
    @GET
    @Cache(maxAge=0)
    public Response hentVarbeidSted(
		@QueryParam("instnr") Long instnr,
		@QueryParam("avdnr") Long avdnr,
		@QueryParam("undavdnr") Long undavdnr,
		@QueryParam("gruppenr") Long gruppenr,
		@QueryParam("format") String format,
		@QueryParam("fra") Integer fraAar,
		@QueryParam("til") Integer tilAar,
		@QueryParam("utplukk") String utplukk,
		@QueryParam("maksantall") Long maksAntall,
		@QueryParam("hovedkategori") String hovedKategori,
		@QueryParam("underkategori") String underKategori,
		@QueryParam("sortering") String sortering
	) {
		if (format == null) {
		    format = "xml";
		} else {
		    format = format.toLowerCase();
		}
		
		if (utplukk == null) {
		    utplukk = "alle";
		} else {
		    utplukk = utplukk.toLowerCase();
		}
		
		if (hovedKategori != null) {
		    hovedKategori = hovedKategori.toUpperCase();
		}
		if (underKategori != null) {
		    underKategori = underKategori.toUpperCase();
		}
	
		if ((instnr == null) ||
		    (!format.equals("xml") && !format.equals("json")) ||
		    (fraAar != null && tilAar != null && fraAar > tilAar) ||
		    (!utplukk.equals("nyeste") && !utplukk.equals("viktigste") &&
		     !utplukk.equals("alle")) ||
		    (maksAntall != null && utplukk.equals("alle")) ||
		    (maksAntall != null && maksAntall < 1)) {
		    return Response.status(400).build();
		}
		if (maksAntall == null || maksAntall > 1000) {
		    maksAntall = 1001L;
		}
	
		String output = null;
		Connection conn = null;
		PreparedStatement stmt;
	
		try {
		    conn = Database.getConnection();
		    String sql;
		    ArrayList<Object> params = new ArrayList<Object>();
	
		    if (utplukk.equals("viktigste")) {
		    	sql =
		    		"SELECT vx.tekst_xml " +
				    "FROM   vitenskapeligarbeid_xml vx, " +
				    "       presentasjon_enhet pe, " +
				    "       presentasjon_varbeid pv, " +
				    "       vitenskapeligarbeid v " +
				    "WHERE  pe.presentasjonslopenr = pv.presentasjonslopenr " +
				    "  AND  pv.varbeidlopenr = v.varbeidlopenr " +
				    "  AND  vx.varbeidlopenr = v.varbeidlopenr " +
				    "  AND  pe.institusjonsnr = ? ";
			} else {
				sql = 
					"SELECT vx.tekst_xml " +
				    "FROM   vitenskapeligarbeid_xml vx, " +
				    "       varbeid_person_sted vps, " +
				    "       vitenskapeligarbeid v " +
				    "WHERE  vx.varbeidlopenr = vps.varbeidlopenr " +
				    "  AND  vx.varbeidlopenr = v.varbeidlopenr " +
				    "  AND  vps.institusjonsnr = ? ";
			    params.add(instnr);
			}
	
		    if (avdnr != null) {
				sql += " and avdnr = ? ";
				params.add(avdnr);
		    }
		    if (undavdnr != null) {
				sql += " and undavdnr = ? ";
				params.add(undavdnr);
		    }
		    if (gruppenr != null) {
				sql += " and gruppenr = ? ";
				params.add(gruppenr);
		    }
	
		    if (fraAar != null) {
				sql += " and v.arstall >= ? ";
				params.add(fraAar);
		    }
		    if (tilAar != null) {
				sql += " and v.arstall <= ? ";
				params.add(tilAar);
		    }
	
		    if (hovedKategori != null) {
				sql += " and v.varbeidhovedkatkode = ? ";
				params.add(hovedKategori);
		    }
		    if (underKategori != null) {
				sql += " and v.varbeidunderkatkode = ? ";
				params.add(underKategori);
		    }
	
		    if (utplukk.equals("nyeste")) {
		    	sql += " order by v.arstall desc ";
		    }
		    if (sortering != null) {
				String sortpred = Filter.lagVarbeidSortering(sortering);
				if (sortpred == null) {
				    return Response.status(400).build();
				}
				if (utplukk.equals("nyeste")) {
				    sql += ", " + sortpred + " ";
				} else {
				    sql += " order by " + sortpred + " ";
				}
		    }
		    // til slutt, begrens antall: (dette maa gjoeres sist)
		    if (maksAntall != null && maksAntall != 0) {
				sql = "select tekst_xml from (" + sql + ") where ROWNUM <= ?";
				params.add(maksAntall);
		    }
	
		    logger.debug(sql);
	
		    stmt = conn.prepareStatement(sql,
						 ResultSet.TYPE_SCROLL_INSENSITIVE,
						 ResultSet.CONCUR_READ_ONLY);
		    
		    int paramIndex = 0;
		    for (Object p : params) {
		    	logger.debug("setter inn object #{} med verdi: <{}>",
				     paramIndex,
				     p.toString());
		    	stmt.setObject(++paramIndex, p);
		    }
	
		    ResultSet rs = stmt.executeQuery();
	
		    rs.last();
		    logger.debug("rs.getRow() == {}", rs.getRow());
		    if (rs.getRow() > 1000) {
		    	return Response.status(413).build();
		    }
		    rs.first();
	
		    ArrayList<String> parts = new ArrayList<String>();
		    while (rs.next()) {
		    	parts.add(rs.getString(1));
		    }
		    if (parts.size() == 0) {
		    	return Response.status(404).build();
		    }
	
		    logger.debug("antall elementer = {}", parts.size());
	
		    XPathExpression xp = null;
		    try {
				xp = Xml.stringToXPath("/frida/forskningsresultat");
				output = Xml.join("http://frida.usit.uio.no/eksport/"
					  + "forskningsresultater/fridaEksport.xsd",
					  xp, parts);
		    } catch (Exception e) {
		    	return Response.status(500).build();
		    }
		    
		} catch (SQLException e) {
		    Database.loggFeil(e);
		    return Response.status(500).build();
		    
		} finally {
		    try {
				if (conn != null) {
				    conn.close();
				}
		    } catch (SQLException e) {
		    	Database.loggFeil(e);
		    	return Response.status(500).build();
		    }
		}
	
		MediaType outputFormat = MediaType.APPLICATION_XML_TYPE;
	
		if (format.equals("json")) {
		    XMLSerializer xs = new XMLSerializer();
		    output = xs.read(output).toString();
		    outputFormat = MediaType.APPLICATION_JSON_TYPE;
		}
	
		return Response
		    .status(200)
		    .entity(output)
		    .variant(new Variant(outputFormat,
					 new java.util.Locale("no_NO"),
					 null))
		    .build();
    }

}
