package no.usit.fridaws;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.sql.DataSource;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Variant;

import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.Date;
import java.util.ArrayList;

import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;

import org.jboss.resteasy.annotations.cache.Cache;

import java.sql.ParameterMetaData;

import java.lang.Integer;

import net.sf.json.xml.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * VarbeiderPerson RPC-tjeneste.
 */
@Path("/hentVarbeiderPerson")
public class VarbeiderPerson {

    private static final Logger logger =
	LoggerFactory.getLogger(VarbeiderPerson.class);

    /**
     * Returnerer Varbeidene til en person basert paa gitte kriterier.
     *
     * @param navn personens brukernavn
     * @param eierkode stedet brukernavnet hører hjemme
     * @param lopenr personens unike loepenummer i databasen
     * @param format "xml" eller "json" for å angi output format
     * @param fra hent kun varbeider utgitt fra og med dette årstallet
     * @param til hent kun varbeider utgitt til og med dette årstallet
     * @param utplukk "alle", "nyeste" eller "viktigste" arbeider
     * @param maksantall begrenser svaret til dette antallet varbeider
     * @param hovedkategori begrens til kun denne hovedkategorien
     * @param underkategori begrens til kun denne underkategorien
     * @param sortering sorteringsangivelse, se bruker-doc.
     * @return en liste med varbeider
     */
    @GET
    @Cache(maxAge=0)
    public Response hentVarbeiderPerson(
		@QueryParam("eierkode") String eierKode,
		@QueryParam("navn") String brukerNavn,
		@QueryParam("lopenr") Long lopenr,
		@QueryParam("format") String format,
		@QueryParam("fra") Integer fraAar,
		@QueryParam("til") Integer tilAar,
		@QueryParam("utplukk") String utplukk,
		@QueryParam("maksantall") Long maksAntall,
		@QueryParam("hovedkategori") String hovedKategori,
		@QueryParam("underkategori") String underKategori,
		@QueryParam("sortering") String sortering
	) {
		if (format == null) {
		    format = "xml";
		} else {
		    format = format.toLowerCase();
		}
	
		if (utplukk == null) {
		    utplukk = "alle";
		} else {
		    utplukk = utplukk.toLowerCase();
		}
	
		if (hovedKategori != null) {
		    hovedKategori = hovedKategori.toUpperCase();
		}
		if (underKategori != null) {
		    underKategori = underKategori.toUpperCase();
		}
	
		if (!InputValidering.okPerson(brukerNavn, eierKode, lopenr) ||
		    !(format.equals("xml") ^ format.equals("json")) ||
		    (fraAar != null && tilAar != null && fraAar > tilAar) ||
		    (!utplukk.equals("nyeste") && !utplukk.equals("viktigste") &&
		     !utplukk.equals("alle")) ||
		    (maksAntall != null && utplukk.equals("alle")) ||
		    (maksAntall != null && maksAntall < 1)) {
		    return Response.status(400).build();
		}
	
		if (brukerNavn != null) {
		    lopenr = Database.getPersonLopeNr(brukerNavn, eierKode);
		}
		if (lopenr == null) {
		    return Response.status(404).build();
		}
	
		String output = null;
		Connection conn = null;
		PreparedStatement stmt;
	
		try {
		    conn = Database.getConnection();
		    String sql;
		    ArrayList<Object> params = new ArrayList<Object>();
	
		    if (utplukk.equals("viktigste")) {
				sql =
					"SELECT vx.tekst_xml " +
				    "FROM   vitenskapeligarbeid_xml vx, " +
				    "       vitenskapeligarbeid v, " +
				    "       presentasjon_varbeid pv, " +
				    "       person p " +
				    "WHERE  p.presentasjonslopenr = pv.presentasjonslopenr " +
				    "  AND  pv.varbeidlopenr = v.varbeidlopenr " +
				    "  AND  vx.varbeidlopenr = v.varbeidlopenr " +
				    "  AND  p.personlopenr = ? ";
		    } else {
				sql = 
					"SELECT vx.tekst_xml " +
				    "FROM   vitenskapeligarbeid_xml vx, " +
				    "       vitenskapeligarbeid v, " +
				    "       varbeid_person vp " +
				    "WHERE  vx.varbeidlopenr = v.varbeidlopenr " +
				    "  AND  vp.varbeidlopenr = v.varbeidlopenr " +
				    "  AND  vp.personlopenr = ? ";
		    }
		    params.add(lopenr);
	
		    if (fraAar != null) {
		    	sql += " AND arstall >= ? ";
				params.add(fraAar);
		    }
		    if (tilAar != null) {
		    	sql += " AND arstall <= ? ";
		    	params.add(tilAar);
		    }
	
		    if (hovedKategori != null) {
				sql += " AND varbeidhovedkatkode = ? ";
				params.add(hovedKategori);
		    }
		    if (underKategori != null) {
				sql += " AND varbeidunderkatkode = ? ";
				params.add(underKategori);
		    }
		    if (utplukk.equals("nyeste")) {
		    	sql += " ORDER BY arstall desc ";
		    }
		    if (sortering != null) {
				String sortpred = Filter.lagVarbeidSortering(sortering);
				if (sortpred == null) {
				    return Response.status(400).build();
				}
				if (utplukk.equals("nyeste")){
				    sql += ", " + sortpred + " ";
				} else {
				    sql += " ORDER BY " + sortpred + " ";
				}
		    }
		    // til slutt, begrens antall: (dette maa gjoeres sist)
		    if (maksAntall != null && maksAntall != 0) {
		    	sql = "SELECT tekst_xml FROM (" + sql + ") WHERE ROWNUM <= ?";
		    	params.add(maksAntall);
		    }
	
		    logger.debug(sql);
			    stmt = conn.prepareStatement(sql);
		    
		    int paramIndex = 0;
		    for (Object p : params) {
		    	logger.debug("setter inn object #{} med verdi: <{}>",
				     paramIndex,
				     p.toString());
		    	stmt.setObject(++paramIndex, p);
		    }
	
		    ResultSet rs = stmt.executeQuery();
	
		    ArrayList<String> parts = new ArrayList<String>();
		    while (rs.next()) {
		    	parts.add(rs.getString(1));
		    }
		    if (parts.size() == 0) {
		    	return Response.status(404).build();
		    }
	
		    logger.debug("antall elementer = {}", parts.size());
	
		    XPathExpression xp = null;
		    try {
		    	xp = Xml.stringToXPath("/frida/forskningsresultat");
		    	output = Xml.join("http://frida.usit.uio.no/eksport/"
					  + "forskningsresultater/fridaEksport.xsd",
					  xp, parts);
		    } catch (Exception e) {
		    	return Response.status(500).build();
		    }
		    
		} catch (SQLException e) {
		    Database.loggFeil(e);
		    return Response.status(500).build();
		    
		} finally {
		    try {
				if (conn != null) {
				    conn.close();
				}
		    } catch (SQLException e) {
		    	Database.loggFeil(e);
		    	return Response.status(500).build();
		    }
		}
	
		MediaType outputFormat = MediaType.APPLICATION_XML_TYPE;
	
		if (format.equals("json")) {
		    XMLSerializer xs = new XMLSerializer();
		    output = xs.read(output).toString();
		    outputFormat = MediaType.APPLICATION_JSON_TYPE;
		}
	
		return Response
		    .status(200)
		    .entity(output)
		    .variant(new Variant(outputFormat,
					 new java.util.Locale("no_NO"),
					 null))
		    .build();
    }

}
